import java.lang.reflect.InvocationHandler;
import java.lang.reflect.Method;
import java.lang.reflect.Proxy;
import java.text.MessageFormat;
import java.util.Locale;
import java.util.ResourceBundle;

import org.eclipse.osgi.util.NLS;

public abstract class MessageFactory {

	public interface I18NResourceBundle {

	}

	public interface I18NNLS {

	}

	@SuppressWarnings("unchecked")
	private static <M> M createResourceBundle(Class<M> clazz) {
		final ResourceBundle bundle = ResourceBundle.getBundle(clazz.getName(), Locale
					.getDefault(), clazz.getClassLoader());

		return (M) Proxy.newProxyInstance(clazz.getClassLoader(),
				new Class<?>[] { clazz }, new InvocationHandler() {

					public Object invoke(Object proxy, Method method,
							Object[] args) throws Throwable {
						if (args == null || args.length == 0) {
							return bundle.getString(method.getName());
						} else {
							return MessageFormat.format(bundle
									.getString(method.getName()), args);
						}

					}

				});
	}

	@SuppressWarnings("unchecked")
	private static <M> M createNLS(Class<M> clazz) {
		try {
			final Class messageClazz = (Class) clazz.getField("NLS_TARGET").get(null);
			NLS.initializeMessages(clazz.getName(), messageClazz);

			if (clazz.isAssignableFrom(messageClazz)) {
				return (M) messageClazz.newInstance();
			} else {
				return (M) Proxy.newProxyInstance(clazz.getClassLoader(),
						new Class<?>[] { clazz }, new InvocationHandler() {

							public Object invoke(Object proxy, Method method,
									Object[] args) throws Throwable {
								String value = (String) messageClazz
										.getField(method.getName()).get(null);

								if (args == null || args.length == 0) {
									return value;
								} else {
									return MessageFormat.format(value, args);
								}

							}
						});

			}
		} catch( Exception e ) {
			throw new RuntimeException();
		}
	}

	public static <M> M create(Class<M> clazz) throws Exception {
		if( I18NNLS.class.isAssignableFrom(clazz) ) {
			return createNLS(clazz);
		} else if( I18NResourceBundle.class.isAssignableFrom(clazz) ) {
			return createResourceBundle(clazz);
		} else {
			throw new RuntimeException();
		}
	}
}
